@@ -5,9 +5,9 @@ This is partly based upon the `MPD protocol documentation
 <http://www.musicpd.org/doc/protocol/>`_, which is a useful resource, but it is
 rather incomplete with regards to data formats, both for requests and
 responses. Thus, we have had to talk a great deal with the the original `MPD
-server <http://mpd.wikia.com/>`_ using telnet to get the details we need to
+server <https://mpd.fandom.com/>`_ using telnet to get the details we need to
 implement our own MPD server which is compatible with the numerous existing
-`MPD clients <http://mpd.wikia.com/wiki/Clients>`_.
+`MPD clients <https://mpd.fandom.com/wiki/Clients>`_.
 """
 
 from __future__ import absolute_import, unicode_literals
@@ -38,7 +38,7 @@ def load_protocol_modules():
 
 
 def INT(value):  # noqa: N802
-    r"""Converts a value that matches [+-]?\d+ into and integer."""
+    r"""Converts a value that matches [+-]?\d+ into an integer."""
     if value is None:
         raise ValueError('None is not a valid integer')
     # TODO: check for whitespace via value != value.strip()?
@@ -54,6 +54,23 @@ def UINT(value):  # noqa: N802
     return int(value)
 
 
+def FLOAT(value):  # noqa: N802
+    r"""Converts a value that matches [+-]\d+(.\d+)? into a float."""
+    if value is None:
+        raise ValueError('None is not a valid float')
+    return float(value)
+
+
+def UFLOAT(value):  # noqa: N802
+    r"""Converts a value that matches \d+(.\d+)? into a float."""
+    if value is None:
+        raise ValueError('None is not a valid float')
+    value = float(value)
+    if value < 0:
+        raise ValueError('Only positive numbers are allowed')
+    return value
+
+
 def BOOL(value):  # noqa: N802
     """Convert the values 0 and 1 into booleans."""
     if value in ('1', '0'):
