@@ -25,18 +25,19 @@ class MpdSession(network.LineProtocol):
             session=self, config=config, core=core, uri_map=uri_map)
 
     def on_start(self):
-        logger.info('New MPD connection from [%s]:%s', self.host, self.port)
+        logger.info('New MPD connection from %s', self.connection)
         self.send_lines(['OK MPD %s' % protocol.VERSION])
 
     def on_line_received(self, line):
-        logger.debug('Request from [%s]:%s: %s', self.host, self.port, line)
+        logger.debug('Request from %s: %s', self.connection, line)
 
         response = self.dispatcher.handle_request(line)
         if not response:
             return
 
         logger.debug(
-            'Response to [%s]:%s: %s', self.host, self.port,
+            'Response to %s: %s',
+            self.connection,
             formatting.indent(self.terminator.join(response)))
 
         self.send_lines(response)
